;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  
;;  The Propeller Blocks from the NSMB series, simplified for SMW
;;  While holding it, Mario jumps higher and falls slower
;;     by leod
;;
;;  Included ExGFX are for SP2, overwrite the smiley coin's top half.
;;
;;  Extra Bit ON will make the sprite boost Mario's jump a lot more,
;;               which is only really usable in vertical levels.
;;  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;Block properties
!BlockPalette = $0C
;sprite palette to use for the block portion, from 08-0F (0C = red, 0D = green, 0B = blue, 0A = yellow)

!BlockPalette2 = $0B
;sprite palette to use for the block portion, from 08-0F (0C = red, 0D = green, 0B = blue, 0A = yellow)
;this one is used when the extra bit is set, so the player knows it acts differently

!BlockSP = $00
;which sprite page the block graphics use (uses vanilla turn block graphics by default)
;SP1/2 = $00,  SP3/4 = $01

;;;;Propeller properties
!PropellerPalette = $0A
;sprite palette to use for the propeller, from 08-0F (0A = yellow, 09 = grey)

!PropellerSP = $00
;which sprite page the propeller graphics use (included ExGFX overwrite smiley coin)
;SP1/2 = $00,  SP3/4 = $01



;;;;;;;;;;;;;
;don't change these defines. or do, but it won't help much, they're just random sprite tables
;;;;;;;;;;;;;

!Extra = $160E        ;extra bit

!Contact = $1570      ;if mario and sprite are making contact, this is nonzero

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite INIT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
STZ !Extra,x
LDA $7FAB10,x        ; check for extra bit
AND #$04             ; (bit 2 -> %100 -> $4)
BEQ NoExtraIsSet     ; if not set, skip the following
LDA #$01
STA !Extra,x
STZ $1504,x

NoExtraIsSet:
RTL
                    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite MAIN
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "MAIN ",pc
PHB                     ; \
PHK                     ;  | The same old
PLB                     ; /
JSR MainCode            ;  Jump to sprite code
PLB                     ; Yep
RTL                     ; Return1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main code 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


  Return11:
RTS

  MainCode:
JSR SubGFX

LDA $9D             ;if sprites are locked, Return1
BNE Return11

JSR SubOffScreen_X3	;handle off screen situation

LDA $14C8,x
CMP #$0B            ;if the sprite isn't in Mario's hands,
BNE .NotGrabbed     ;check if it should be and act solid

LDA #$08
STA $1504,x	    ; TIME SHENANS

LDA $77
AND #$04            ;if not on ground
BNE .NoLow

;low g
LDA $7D
BMI .Rise           ;if rising, just jump higher

LDA $15
AND #$80            ;check if B is pressed
BEQ .HardDesc       ;if yes, dont go there

LDA $7D
CMP #$0A            ;if not reached slow descent speed yet, do the hard desc
BCC .Rise

LDA #$0A
STA $7D             ;if holding B, soft descent
BRA .NoLow

  .HardDesc
LDA $7D
DEC
DEC
STA $7D

LDA !Extra,x
BNE .Rise           ;if extra bit isn't set, handle falling softly more here
DEC $7D

  .Rise
DEC $7D

LDA !Extra,x
BEQ .NoLow          ;if extra bit is set, handle falling and rising softly more here
DEC $7D
BRA .NoLow2

  .NoLow
LDA $14
AND #$01
BNE .NoLow2
DEC $7D
  .NoLow2
RTS


  .NotGrabbed

LDA $1504,x
BEQ .NoTimeForTime
DEC
STA $1504,x

  .NoTimeForTime

LDA $AA,x
BMI .NotStill

LDA #$08            ;normal state
STA $14C8,x

  .NotStill
LDA $14C8,x
CMP #$08            ;only do this in standard state
BEQ .NoSuchContact
JMP Contact
.NoSuchContact

JSL $019138         ;interact with objects
LDA $B6,x
BEQ .DoneX
BMI .GoingLeft

DEC $B6,x

BRA .DoneX

  .GoingLeft

INC $B6,x

  .DoneX
LDA $1588,x
AND #$08            ;check for collision with the ceiling
BEQ .NoCeiling

LDA #$0A
STA $AA,x
BRA .NoFallThisFrame

  .NoCeiling
LDA $1588,x
AND #$04            ;check for collision with the floor
BEQ .NoYRebound     ;if sprite is on the floor..

LDA $AA,x
CMP #$05
BCS .invert
STZ $AA,x
STZ $B6,x
BRA .NoFallThisFrame
BRA .NoYRebound
.invert
LSR
EOR #$FF
STA $AA,x

  .NoYRebound
LDA $14
AND #$03
BNE .NoFallThisFrame

LDA $AA,x
INC
CMP #$1A
BMI .StoreY
BCS .NoFallThisFrame

  .StoreY
STA $AA,x

  .NoFallThisFrame


JSL $01801A         ;update x
JSL $018022         ;update y

;mario<->sprite
  Contact:
JSL $03B69F         ;fill this sprite into clipping slot A
JSL $03B664
JSL $03B72B
		      ;if mario isn't touching the sprite, don't check for carrying
BCC .NoContact

LDA $1504,x
BNE .NoContact

LDA $148F
ORA $1470           ;if mario is already carrying a sprite
ORA $187A           ;or riding yoshi, do nothing further
ORA $140D           ;or spinjumping
BNE .NoContact

LDA $15
AND #$40            ;check for X and Y being pressed
BEQ .NoContact      ;if not, return

LDA #$01
STA $148F
STA $1470

LDA #$0B
STA $14C8,x

BRA .NoContact

  .NoContact
RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Graphics Routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  TileSize:
db $00,$00  ;propeller
db $02      ;body

  TileXOff:
db $00,$08  ;propeller
db $00      ;body

  TileYOff:
db $F8,$F8  ;propeller
db $00      ;body

  TileGFX:
db $C2,$C3  ;propeller
db $40      ;body

  TileProps:
db !PropellerPalette-$08<<1|$20|!PropellerSP,!PropellerPalette-$08<<1|$60|!PropellerSP  ;propeller (contains palette, flip and priority)
db $20|!BlockSP  ;body (contains priority)

  BlockProps:
db !BlockPalette-$08<<1,!BlockPalette2-$08<<1   ;(contains block palette)

  SubGFX:
JSR GET_DRAW_INFO
  PHX


LDX #$02
  .PropellerLoop
PHX

CPX #$02        ;if X is 2 aka the body, just keep X, otherwise do this special shit
BEQ .SkipPropelTile

LDA $14
LSR             ;more LSR = slower animation
LSR
AND #$01        ;to get the propeller animation
TAX
  .SkipPropelTile

LDA TileGFX,x   ;load tile
STA $0302,y     ;store it as normal

PLX             ;restore x if we messed with it
PHX

CPX #$02        ;again check if it's the body, but this time skip if it is not
BNE .SkipBlockPal

LDX $15E9
LDA !Extra,x    ;get extra bit in x
TAX
LDA BlockProps,x;and load appropriate palette
PLX             ;get back loop x
PHX
ORA TileProps,x ;and add the Priority
STA $0303,y
BRA .PropsDone

  .SkipBlockPal
LDA TileProps,x ;load props
STA $0303,y

  .PropsDone
LDA $00         ;load sprite x pos
CLC
ADC TileXOff,x
STA $0300,y     ;store to oam x pos

LDA $01         ;load sprite y pos
CLC
ADC TileYOff,x
STA $0301,y     ;store to oam y pos


TYA
LSR A
LSR A
PHY
TAY
LDA TileSize,x ;load size
STA $0460,y    ;store to size table
PLY

INY : INY : INY : INY

PLX
DEX
BPL .PropellerLoop

  PLX

LDY #$FF        ; #$00 = 8x8, #$02 = 16x16, #$FF = variable
LDA #$02        ; tiles drawn; #$00 = 1, #$01 = 2, etc.
JSL $01B7B3     ; handle oam
RTS

;;;;;
; LEOD'S SOLID SPRITE ROUTINE
; ASSUMES THAT SPRITE SLOT A IS FILLED IN
; Which side Mario is touching the sprite from is stored to $1570,x
; Values are: 00 = no contact; 01 = top; 02 = bottom; FE = left; FF = right
;;;;;

SolidContact:
JSL $03B664           ;fill in mario clipping

STZ $1570,x           ;no contact ram
JSL $03B72B           ;check if the filled in objects intersect
BCS .Contact          ;go to NoContact if no contact
RTS
  .Contact


;check if mario should ride the sprite before the edges so he isnt
;stuck on top because hes "inside" the sprite technically
JSR Get_Solid_Vert
LDY $187A               ;load yoshi flag into y
LDA $0F
CMP .HeightReq,y
BCS .DecideDownOrSides  ;if difference is less than FF-E6, mario isnt quite high enough
BPL .DecideDownOrSides  ;if difference is positive, mario is -below-

INC $1570,x             ;tell sprite that mario is touching from above (01)
                        ;INC works because it's re-set to 00 every frame

LDA #$01
STA $1471               ;set "on sprite" flag

LDA #$E3                ;load E3 (this seems to just randomly be different between sprites? some vanilla ones use E6, others E1, this one needs E3)
LDY $187A
BEQ .NoYoshi
LDA #$D3                ;if on yoshi, load D3 instead (one block higher)
  .NoYoshi
SEC
SBC #$01                ;substract one to align mario
CLC
ADC $05                 ;add sprite y position, low byte
STA $96                 ;store to mario y position, low byte
LDA $0B                 ;load sprite y position, high byte
ADC #$FF                ;decrease by 1 or keep same depending on overflow
STA $97                 ;store to mario y position

RTS


.HeightReq        ;height difference requirement with and without yoshi
db $D4,$C6,$C6

  .DecideDownOrSides
LDA $0A
XBA               ;load left edge of sprite
LDA $04
REP #$20
SEC
SBC #$000C        ;take into account right edge
CMP $D1           ;compare to mario
SEP #$20
BCS .HorzCheck

REP #$20
CLC
ADC #$0008        ;get back left edge of sprite
STA $0C
SEP #$20

LDA $0C
CLC
ADC $06           ;add width to get right edge
STA $0C
LDA $0D
ADC #$00
STA $0D
REP #$20
LDA $0C
CMP $D1           ;compare to mario
SEP #$20
BCC .HorzCheck

;mario is touching the sprite from below
LDA #$02
STA $1570,x       ;tell sprite that mario is touching from below (02)

JSR Get_Solid_Horz
LDA $0F                 ;get range difference between sprite and mario
BMI .left

.right
REP #$20
LDA $94
CLC
ADC #$0001      ;move player left by 01
STA $94
SEP #$20
RTS

.left
REP #$20
LDA $94
SEC
SBC #$0001      ;move player left by 01
STA $94
SEP #$20
RTS

  .HorzCheck
;check if mario -was- outside the sprite horizontally
JSR Get_Solid_Horz
LDA $0F                 ;get range difference between sprite and mario
BMI .MarioLeft
;mario is to right of sprite

SEC
SBC $06                 ;substract width for left edge
BPL .NoContact

;mario is touching right
LDA #$FE
STA $1570,x        ;tell sprite mario is touching right (FE)

LDA $04
SEC
SBC #$02
CLC
ADC $06
STA $0C

LDA $0A
ADC #$00
STA $0D

REP #$20
LDA $0C
STA $94
SEP #$20


LDA $7B
BPL .NoContact      ;unless mario is moving left, zero out speed
STZ $7B

RTS

  .MarioLeft
;mario is to left of sprite
CLC
ADC #$10          ;add mario width
BMI .NoContact

;mario is touching left
DEC $1570,x       ;tell sprite mario is touching left (FF)
                  ;dec works because it's re-set to 00 every frame

LDA $04
SEC
SBC #$0E        ;SBC #$0D
STA $0C

LDA $0A
SBC #$00
STA $0D

REP #$20
LDA $0C
STA $94
SEP #$20

LDA $7B
BMI .NoContact      ;unless mario is moving right, zero out speed
STZ $7B

  .NoContact
RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
        ;org $03B75C
 
TABLE1:              db $0C,$1C
TABLE2:              db $01,$02
 
GET_DRAW_INFO:       STZ $186C,x             ; reset sprite offscreen flag, vertical
        STZ $15A0,x             ; reset sprite offscreen flag, horizontal
        LDA $E4,x               ; \
        CMP $1A                 ;  | set horizontal offscreen if necessary
        LDA $14E0,x             ;  |
        SBC $1B                 ;  |
        BEQ ON_SCREEN_X         ;  |
        INC $15A0,x             ; /
 
ON_SCREEN_X:         LDA $14E0,x             ; \
        XBA      ;  |
        LDA $E4,x               ;  |
        REP #$20                ;  |
        SEC      ;  |
        SBC $1A                 ;  | mark sprite INVALID if far enough off screen
        CLC      ;  |
        ADC.w #$0040            ;  |
        CMP.w #$0180            ;  |
        SEP #$20                ;  |
        ROL A                   ;  |
        AND #$01                ;  |
        STA $15C4,x             ; /
        BNE INVALID             ;
       
        LDY #$00                ; \ set up LOOP:
        LDA $1662,x             ;  |
        AND #$20                ;  | if not smushed (1662 & 0x20), go through LOOP twice
        BEQ ON_SCREEN_LOOP      ;  | else, go through LOOP once
        INY      ; /
ON_SCREEN_LOOP:      LDA $D8,x               ; \
        CLC      ;  | set vertical offscreen if necessary
        ADC TABLE1,y            ;  |
        PHP      ;  |
        CMP $1C                 ;  | (vert screen boundry)
        ROL $00                 ;  |
        PLP      ;  |
        LDA $14D4,x             ;  |
        ADC #$00                ;  |
        LSR $00                 ;  |
        SBC $1D                 ;  |
        BEQ ON_SCREEN_Y         ;  |
        LDA $186C,x             ;  | (vert offscreen)
        ORA TABLE2,y            ;  |
        STA $186C,x             ;  |
ON_SCREEN_Y:         DEY         ;  |
        BPL ON_SCREEN_LOOP      ; /
 
        LDY $15EA,x             ; get offset to sprite OAM
        LDA $E4,x               ; \
        SEC      ;  |
        SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
        STA $00                 ; /
        LDA $D8,x               ; \
        SEC      ;  |
        SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
        STA $01                 ; /
        RTS      ; Return1
 
INVALID:             PLA         ; \ Return1 from *main gfx routine* subroutine...
        PLA      ;  |    ...(not just this subroutine)
        RTS      ; /

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D and SLIGHTLY edited to work with carryable sprites
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                   
SPR_T12:            db $40,$C0
SPR_T13:            db $01,$FE
SPR_T14:            db $30,$C0,$A0,$C0,$A0,$F0,$60,$90                    ;bank 1 sizes
                                db $30,$C0,$A0,$80,$A0,$40,$60,$B0                    ;bank 3 sizes
SPR_T15:            db $01,$FF,$01,$FF,$01,$FF,$01,$FF                    ;bank 1 sizes
                                                db $01,$FF,$01,$FF,$01,$00,$01,$FF                    ;bank 3 sizes
 
SubOffScreen_X1:  LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SubOffScreen_X2:  LDA #$04                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X3:  LDA #$06                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X4:  LDA #$08                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X5:  LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X6:  LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X7:  LDA #$0E                ;  |
STORE_03:                    STA $03                                            ;  |            
                                                BRA START_SUB                           ;  |
SubOffScreen_X0:  STZ $03                                               ; /
 
START_SUB:          JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, Return1
                    BEQ Return1_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /    
                    LDA $D8,x               ; \
                    CLC                     ; |
                    ADC #$50                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; |
                    CMP #$02                ; |
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, Return1
                    AND #$04                ; |
                    BNE Return1_35           ; /
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31:            LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL Return1_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE:       BRA KILL_SPRITE         ; /  
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE:        STZ $14C8,x             ; erase sprite
Return1_35:          RTS                     ; Return1
 
VERTICAL_LEVEL:     LDA $167A,x             ; \ if "process offscreen" flag is set, Return1
                    AND #$04                ; |
                    BNE Return1_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; |
                    LSR A
                    BCS Return1_35           ; /
 
                    LDA $E4,x               ; \
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38:            LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL Return1_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490
 
SUB_IS_OFF_SCREEN:
                    LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0
                    ORA $186C,x             ; |  
                    RTS                     ; / Return1


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; vertical mario/sprite position check
; changed for a specific solid sprite routine to use filled-in clip values
; Y = 1 if mario above sprite
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                ;org $03B829 

Get_Solid_Vert:       
LDY #$00        ;init as 0
LDA $D3         ;mario y pos
SEC
SBC #$10
SBC $05         ;minus sprite y pos
STA $0F         ;store range difference in 0F
LDA $D4         ;mario y pos high
SBC $0B         ;minus sprite y pos high + overflow
BPL MarioLower  ;if the result is not positive (so Mario would be below)
INY             ;increase y
  MarioLower:
STY $0E         ;store in $0E
RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; horizontal mario/sprite position check
; changed for a specific solid sprite routine to use filled-in clip values
; Y = 1 if mario left of sprite
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Get_Solid_Horz:
LDY #$00        ;init as 0
LDA $D1         ;mario x pos
SEC
SBC $04         ;minus sprite x pos
STA $0F         ;store range difference in 0F
LDA $D2         ;mario x pos high
SBC $0A         ;minus sprite x pos high + overflow
BPL MarioRight  ;if the result is not positive (so Mario would be to the right)
INY             ;increase y
  MarioRight:
STY $0E         ;store in $0E
RTS